<?php

namespace App\Http\Livewire\Tables;

use App\Helpers\LogHelper;
use Exception;
use App\Models\Task;
use App\Models\User;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use App\Services\TaskReminderService;
use Illuminate\Database\Eloquent\Builder;
use Rappasoft\LaravelLivewireTables\Views\Column;
use Rappasoft\LaravelLivewireTables\DataTableComponent;

class TaskDatatables extends DataTableComponent
{
    protected $model = Task::class;

    // public $user;

    public function configure(): void
    {
        $this->setPrimaryKey('id');
        $this->setAdditionalSelects(['tasks.is_published as is_published']);
        $this->setEagerLoadAllRelationsEnabled();
    }

    public function mount($param)
    {
        $this->query_param = $param;
    }

    public function getUser()
    {
        $user = User::findOrFail(Auth::id());
        return $user;
    }

    public function builder(): Builder
    {
        $param = $this->query_param;
        // dd($this->query_param);
        if ($this->getUser()->hasPermissionTo('manage tasks')) {
            return Task::query()->orderBy('id', 'desc');

        } else {
            return Task::query()->whereHas('users', function ($query) {
                return $query->where('user_id', Auth::id())
                    ->where('is_published', true);
            })
            ->whereHas('usersTask', function ($query) use ($param) {
                if ($param == 'all') {
                    return $query->where('user_id', Auth::id());
                } else {
                    return $query->where('user_id', Auth::id())
                        ->where('completed', $param);
                }
            })
            ->orderBy('id', 'desc');
        }
    }

    public function show($id)
    {
        return redirect()->route('tasks.show', ['id' => $id]);
    }

    public function edit($id)
    {
        return redirect()->route('tasks.edit', ['id' => $id]);
    }

    public function delete($id, TaskReminderService $taskReminderService)
    {
        $task = Task::findOrFail($id);
        $task_name = $task->title;

        if (Auth::user()->userData) {
            $deleted_by = Auth::user()->userData->fullName();
        } else {
            $deleted_by = Auth::user()->username;
        }

        $task->update([
            'deleted_by' => $deleted_by,
            'is_published' => false
        ]);
        $taskReminderService->disable_jobs($task);
        $task->delete();

        $message = "Task $task_name is archived Successfully";
        $data = ['task' => $task];
        LogHelper::logInfo($message, $data);

        return redirect()->to('/tasks')->with('message', $message);
    }

    public function publish($id, TaskReminderService $taskReminderService)
    {

        $task = Task::with('documents')->findorFail($id);
        DB::beginTransaction();
        try {
            if ($task->is_published) {
                $task->update(['is_published' => false]);
                $taskReminderService->disable_jobs($task);

                $message = "Task $task->title is unpublished";
                $style = 'info';
            } else {
                $task->update(['is_published' => true]);
                $taskReminderService->enable_jobs($task);

                $message = "Task $task->title is published";
                $style = 'success';
            }

            DB::commit();

            $data = ['task' => $task];
            LogHelper::logInfo($message, $data);

            $this->dispatchBrowserEvent('notifyUser', [
                'style' => $style,
                'status' =>  __('messages.Info'),
                'message' => $message
            ]);
        } catch (\Exception $e) {
            DB::rollback();
            $this->dispatchBrowserEvent('notifyUser', [
                'style' => 'danger',
                'status' =>  __('messages.Error'),
                'message' => "There was an error, please try again!"
            ]);

            $message = $e->getMessage();
            $data = ['task' => $task, 'error' => $e->getMessage()];
            LogHelper::logMessageError($message, $data);
        }
    }

    public function columns(): array
    {

        $columns = [
            Column::make(__('messages.Id'), "id")
                ->sortable(),
            Column::make(__('messages.Title'), "title")
                ->sortable()
                ->searchable(),
            Column::make(__('messages.Description'), "description")
                ->sortable()
                ->searchable(),
            Column::make(__('messages.Due date'), "due_date")
                ->sortable(),
            Column::make(__('messages.Created at'), "created_at")
                ->sortable(),
            Column::make(__('messages.Updated at'), "updated_at")
                ->sortable(),
        ];

        $publish_column = [
            Column::make(__('messages.Published'))
                ->label(
                    function ($row, Column $column) {

                        if ($row->is_published == 1) {
                            $is_checked = "checked";
                        } else {
                            $is_checked = "";
                        }
                        $publish = '<label class="inline-flex relative items-center cursor-pointer">
                              <input wire:click="publish(' . $row->id . ')"
                                     ' . $is_checked . '
                                     type="checkbox"
                                     name="toggle" id="toggle"
                                     class="sr-only peer">
                              <div  class="w-11 h-6 bg-gray-200 peer-focus:outline-none peer-focus:ring-4 peer-focus:ring-blue-300 dark:peer-focus:ring-blue-800 rounded-full peer dark:bg-gray-400 peer-checked:after:translate-x-full peer-checked:after:border-white after:content-[' . "''" . '] after:absolute after:top-[2px] after:left-[2px] after:bg-white after:border-gray-300 after:border after:rounded-full after:h-5 after:w-5 after:transition-all dark:border-gray-600 peer-checked:bg-blue-600"></div>
                            </label>';

                        return $publish;
                    }
                )->html(),
        ];

        $admin_columns = [

            Column::make(__('messages.Actions'))
                ->label(
                    function ($row, Column $column) {
                        $delete = '
                            <button
                                class="inline-flex items-center text-white bg-gray-600 hover:bg-gray-700 focus:outline-none focus:ring-4 focus:ring-gray-300 font-medium rounded-full text-sm h-10 px-5 m-2 text-center dark:bg-gray-900 dark:hover:bg-gray-800 dark:focus:ring-gray-900"
                                type="button"
                                data-modal-toggle="delete-modal_' . $row->id . '"
                                >
                                <svg xmlns="http://www.w3.org/2000/svg" class="h-5 w-5 mr-1" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                                    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M5 8h14M5 8a2 2 0 110-4h14a2 2 0 110 4M5 8v10a2 2 0 002 2h10a2 2 0 002-2V8m-9 4h4" />
                                </svg>
                               ' . __('messages.Archive') . '
                            </button>
                            <div id="delete-modal_' . $row->id . '" tabindex="-1" class="hidden overflow-y-auto overflow-x-hidden fixed top-0 right-0 left-0 z-50 p-4 md:inset-0 h-modal md:h-full">
                                <div class="relative w-full max-w-md h-full md:h-auto">
                                    <div class="relative bg-white rounded-lg shadow dark:bg-gray-700">
                                        <button type="button" class="absolute top-3 right-2.5 text-gray-400 bg-transparent hover:bg-gray-200 hover:text-gray-900 rounded-lg text-sm p-1.5 ml-auto inline-flex items-center dark:hover:bg-gray-800 dark:hover:text-white" data-modal-toggle="delete-modal_' . $row->id . '">
                                            <svg aria-hidden="true" class="w-5 h-5" fill="currentColor" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" d="M4.293 4.293a1 1 0 011.414 0L10 8.586l4.293-4.293a1 1 0 111.414 1.414L11.414 10l4.293 4.293a1 1 0 01-1.414 1.414L10 11.414l-4.293 4.293a1 1 0 01-1.414-1.414L8.586 10 4.293 5.707a1 1 0 010-1.414z" clip-rule="evenodd"></path></svg>
                                            <span class="sr-only">
                                            ' . __('messages.Close modal') . '
                                            </span>
                                        </button>
                                        <div class="p-6 text-center">
                                            <svg aria-hidden="true" class="mx-auto mb-4 w-14 h-14 text-gray-400 dark:text-gray-200" fill="none" stroke="currentColor" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M12 8v4m0 4h.01M21 12a9 9 0 11-18 0 9 9 0 0118 0z"></path></svg>
                                            <h3 class="mb-5 text-lg font-normal text-gray-500 dark:text-gray-400">
                                            ' . __('messages.Are you sure you want to archive this task?') . '
                                            </h3>
                                            <button data-modal-toggle="delete-modal_' . $row->id . '" type="button" class="text-white bg-red-600 hover:bg-red-800 focus:ring-4 focus:outline-none focus:ring-red-300 dark:focus:ring-red-800 font-medium rounded-lg text-sm inline-flex items-center px-5 py-2.5 text-center mr-2" wire:click="delete(' . $row->id . ')">
                                                ' . __('messages.Yes, I am sure') . '
                                            </button>
                                            <button data-modal-toggle="delete-modal_' . $row->id . '" type="button" class="text-gray-500 bg-white hover:bg-gray-100 focus:ring-4 focus:outline-none focus:ring-gray-200 rounded-lg border border-gray-200 text-sm font-medium px-5 py-2.5 hover:text-gray-900 focus:z-10 dark:bg-gray-700 dark:text-gray-300 dark:border-gray-500 dark:hover:text-white dark:hover:bg-gray-600 dark:focus:ring-gray-600">
                                            ' . __('messages.No, cancel') . '
                                            </button>
                                        </div>
                                    </div>
                                </div>
                            </div>';
                        $show = '
                            <button
                                class="inline-flex items-center text-white bg-blue-700 hover:bg-blue-800 focus:outline-none focus:ring-4 focus:ring-blue-300 font-medium rounded-full text-sm h-10 px-5 m-2 text-center dark:bg-blue-600 dark:hover:bg-blue-700 dark:focus:ring-blue-800"
                                type="button"
                                wire:click="show(' . $row->id . ')"
                                >
                                <svg class="h-5 w-5 mr-1" fill="currentColor" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg">
                                    <path d="M10 12a2 2 0 100-4 2 2 0 000 4z"></path><path fill-rule="evenodd" d="M.458 10C1.732 5.943 5.522 3 10 3s8.268 2.943 9.542 7c-1.274 4.057-5.064 7-9.542 7S1.732 14.057.458 10zM14 10a4 4 0 11-8 0 4 4 0 018 0z" clip-rule="evenodd"></path>
                                </svg>
                                ' . __('messages.View') . '
                            </button>';
                        $edit = '
                            <button
                                class="inline-flex items-center text-white bg-green-700 hover:bg-green-800 focus:outline-none focus:ring-4 focus:ring-green-300 font-medium rounded-full text-sm h-10 px-5 m-2 text-center dark:bg-green-600 dark:hover:bg-green-700 dark:focus:ring-green-800"
                                type="button"
                                wire:click="edit(' . $row->id . ')"
                                >
                                <svg fill="#ffffff" xmlns="http://www.w3.org/2000/svg" class="h-5 w-5 mr-1" viewBox="0 0 32 32" width="20px" height="20px">
                                    <path d="M 23.90625 3.96875 C 22.859375 3.96875 21.8125 4.375 21 5.1875 L 5.1875 21 L 5.125 21.3125 L 4.03125 26.8125 L 3.71875 28.28125 L 5.1875 27.96875 L 10.6875 26.875 L 11 26.8125 L 26.8125 11 C 28.4375 9.375 28.4375 6.8125 26.8125 5.1875 C 26 4.375 24.953125 3.96875 23.90625 3.96875 Z M 23.90625 5.875 C 24.410156 5.875 24.917969 6.105469 25.40625 6.59375 C 26.378906 7.566406 26.378906 8.621094 25.40625 9.59375 L 24.6875 10.28125 L 21.71875 7.3125 L 22.40625 6.59375 C 22.894531 6.105469 23.402344 5.875 23.90625 5.875 Z M 20.3125 8.71875 L 23.28125 11.6875 L 11.1875 23.78125 C 10.53125 22.5 9.5 21.46875 8.21875 20.8125 Z M 6.9375 22.4375 C 8.136719 22.921875 9.078125 23.863281 9.5625 25.0625 L 6.28125 25.71875 Z"/>
                                </svg>
                                ' . __('messages.Edit') . '
                            </button>';
                        if ($this->getUser()->hasPermissionTo('manage tasks')) {
                            return $edit . $delete;
                        } else {
                            return $show;
                        }
                    }
                )->html(),
        ];

        if ($this->getUser()->hasPermissionTo('manage tasks')) {
            $return_columns = array_merge($columns, $publish_column, $admin_columns);
        } else {
            $return_columns = array_merge($columns, $admin_columns);
        }

        return $return_columns;
    }
}
